@@ -1,6 +1,7 @@
 /*
  *  ClamdTOP
  *
+ *  Copyright (C) 2015, 2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008 - 2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -57,6 +58,7 @@
 #include <assert.h>
 #include <errno.h>
 
+#include "libclamav/clamav.h"
 #include "shared/optparser.h"
 #include "shared/misc.h"
 
@@ -101,6 +103,9 @@ static void cleanup(void);
 static int send_string_noreconn(conn_t *conn, const char *cmd);
 static void send_string(conn_t *conn, const char *cmd);
 static int read_version(conn_t *conn);
+char *get_ip(const char *ip);
+char *get_port(const char *ip);
+char *make_ip(const char *host, const char *port);
 
 enum exit_reason {
         FAIL_CMDLINE=1,
@@ -492,8 +497,8 @@ static void print_con_info(conn_t *conn, const char *fmt, ...)
 	va_start(ap, fmt);
 	if (stats_head_window) {
 		char *buf = malloc(maxx);
-		memset(buf, ' ', maxx);
 		OOM_CHECK(buf);
+		memset(buf, ' ', maxx);
 		vsnprintf(buf, maxx-1, fmt, ap);
 		buf[strlen(buf)] = ' ';
 		buf[maxx-1] = '\0';
@@ -509,7 +514,7 @@ static void print_con_info(conn_t *conn, const char *fmt, ...)
 
 char *get_ip(const char *ip)
 {
-    char *dupip, *p1, *p2;
+    char *dupip, *p1;
     unsigned int i;
 
     /*
@@ -605,14 +610,15 @@ char *make_ip(const char *host, const char *port)
 
 static int make_connection_real(const char *soname, conn_t *conn)
 {
-    int s;
+    int s = -1;
     struct timeval tv;
     char *port=NULL;
-    char *name, *pt = strdup(soname);
+    char *pt = strdup(soname);
     const char *host = pt;
     struct addrinfo hints, *res=NULL, *p;
     int err;
 
+    OOM_CHECK(pt);
     conn->tcp = 0;
 
 #ifndef _WIN32
@@ -1083,11 +1089,13 @@ static void parse_stats(conn_t *conn, struct stats *stats, unsigned idx)
 
 	if (!conn->version) {
 		stats->engine_version = strdup("???");
+		OOM_CHECK(stats->engine_version);
 		return;
 	}
 	p = pstart = vstart = strchr(conn->version, ' ');
 	if (!vstart) {
 	    stats->engine_version = strdup("???");
+	    OOM_CHECK(stats->engine_version);
 	    return;
 	}
 	/* find digit in version */
@@ -1109,9 +1117,10 @@ static void parse_stats(conn_t *conn, struct stats *stats, unsigned idx)
 	stats->engine_version[p-pstart] = '\0';
 
 	pstart = strchr(p, '/');
-	if (!pstart)
+	if (!pstart) {
 		stats->db_version = strdup("????");
-	else {
+		OOM_CHECK(stats->db_version);
+	} else {
 		pstart++;
 		p = strchr(pstart, '/');
 		if (!p)
@@ -1217,22 +1226,25 @@ static int read_version(conn_t *conn)
 
 static void sigint(int a)
 {
+    UNUSEDPARAM(a);
 	EXIT_PROGRAM(SIGINT_REASON);
 }
 
 static void help(void)
 {
     printf("\n");
-    printf("           Clam AntiVirus: Monitoring Tool %s\n", get_version());
-    printf("           By The ClamAV Team: http://www.clamav.net/team\n");
-    printf("           (C) 2008-2009 Sourcefire, Inc.\n\n");
-    printf("clamdtop [-hVc] [host[:port] /path/to/clamd.socket ...]\n\n");
-    printf("    --help                 -h         Show help\n");
+    printf("                       Clam AntiVirus: Monitoring Tool %s\n", get_version());
+    printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
+    printf("           (C) 2008-2018 Cisco Systems, Inc.\n");
+    printf("\n");
+    printf("    clamdtop [-hVc] [host[:port] /path/to/clamd.socket ...]\n");
+    printf("\n");
+    printf("    --help                 -h         Show this help\n");
     printf("    --version              -V         Show version\n");
     printf("    --config-file=FILE     -c FILE    Read clamd's configuration files from FILE\n");
-    printf("    --defaultcolors	       -d	  Use default terminal colors\n");
-    printf("	host[:port]			  Connect to clamd on host at port (default 3310)\n");
-    printf("    /path/to/clamd.socket		  Connect to clamd over a local socket\n");
+    printf("    --defaultcolors	       -d         Use default terminal colors\n");
+    printf("    host[:port]                       Connect to clamd on host at port (default 3310)\n");
+    printf("    /path/to/clamd.socket             Connect to clamd over a local socket\n");
     printf("\n");
     return;
 }
