@@ -123,8 +123,8 @@ int32_t cli_bcapi_seek(struct cli_bc_ctx* ctx, int32_t pos, uint32_t whence)
             return -1;
     }
     if (off < 0 || off > ctx->file_size) {
-        cli_dbgmsg("bcapi_seek: out of file: %ld (max %d)\n",
-                   off, ctx->file_size);
+        cli_dbgmsg("bcapi_seek: out of file: %lld (max %d)\n",
+                   (long long)off, ctx->file_size);
         return -1;
     }
     cli_event_int(EV, BCEV_OFFSET, off);
@@ -251,10 +251,10 @@ uint32_t cli_bcapi_trace_scope(struct cli_bc_ctx *ctx, const uint8_t *scope, uin
     if (ctx->scope != (const char*)scope) {
         ctx->scope = (const char*)scope ? (const char*)scope : "?";
         ctx->scopeid = scopeid;
-        ctx->trace_level |= 0x80;/* temporarely increase level to print params */
+        ctx->trace_level |= 0x80;/* temporarily increase level to print params */
     } else if ((ctx->trace_level >= trace_scope) && ctx->scopeid != scopeid) {
         ctx->scopeid = scopeid;
-        ctx->trace_level |= 0x40;/* temporarely increase level to print location */
+        ctx->trace_level |= 0x40;/* temporarily increase level to print location */
     }
     return 0;
 }
@@ -525,13 +525,13 @@ int32_t cli_bcapi_extract_new(struct cli_bc_ctx *ctx, int32_t id)
     cli_dbgmsg("bytecode: scanning extracted file %s\n", ctx->tempfile);
     cctx = (cli_ctx*)ctx->ctx;
     if (cctx) {
-        cli_file_t current = cctx->container_type;
-        if (ctx->containertype != CL_TYPE_ANY)
-            cctx->container_type = ctx->containertype;
         cctx->recursion++;
+        if (ctx->containertype != CL_TYPE_ANY) {
+            size_t csize = cli_get_container_size(cctx, -2);
+            cli_set_container(cctx, ctx->containertype, csize);
+        }
         res = cli_magic_scandesc(ctx->outfd, cctx);
         cctx->recursion--;
-        cctx->container_type = current;
         if (res == CL_VIRUS) {
             ctx->virname = cli_get_last_virus(cctx);
             ctx->found = 1;
@@ -811,8 +811,19 @@ int32_t cli_bcapi_inflate_init(struct cli_bc_ctx *ctx, int32_t from, int32_t to,
         cli_dbgmsg("bytecode api: inflate_init: invalid buffers!\n");
         return -1;
     }
-    memset(&stream, 0, sizeof(stream));
-    ret = inflateInit2(&stream, windowBits);
+    b = cli_realloc(ctx->inflates, sizeof(*ctx->inflates)*n);
+    if (!b) {
+        return -1;
+    }
+    ctx->inflates = b;
+    ctx->ninflates = n;
+    b = &b[n-1];
+
+    b->from = from;
+    b->to = to;
+    b->needSync = 0;
+    memset(&b->stream, 0, sizeof(stream));
+    ret = inflateInit2(&b->stream, windowBits);
     switch (ret) {
         case Z_MEM_ERROR:
             cli_dbgmsg("bytecode api: inflateInit2: out of memory!\n");
@@ -830,19 +841,6 @@ int32_t cli_bcapi_inflate_init(struct cli_bc_ctx *ctx, int32_t from, int32_t to,
             return -1;
     }
 
-    b = cli_realloc(ctx->inflates, sizeof(*ctx->inflates)*n);
-    if (!b) {
-        inflateEnd(&stream);
-        return -1;
-    }
-    ctx->inflates = b;
-    ctx->ninflates = n;
-    b = &b[n-1];
-
-    b->from = from;
-    b->to = to;
-    b->needSync = 0;
-    memcpy(&b->stream, &stream, sizeof(stream));
     return n-1;
 }
 
