@@ -1,7 +1,13 @@
 /*
+ *  Copyright (C) 2015, 2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2010 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
+ * 
+ *  Summary: Code to parse Clamav CVD database format.
+ * 
+ *  Acknowledgements: ClamAV untar code is based on a public domain minitar utility
+ *                    by Charles G. Waldman.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -132,7 +138,6 @@ static int cli_untgz(int fd, const char *destdir)
 		    cli_untgz_cleanup(path, infile, outfile, fdd);
 		    return -1;
 	    }
-	    in_block = 1;
 
 	    if(outfile) {
 		if(fclose(outfile)) {
@@ -159,6 +164,9 @@ static int cli_untgz(int fd, const char *destdir)
 		return -1;
 	    }
 
+	    if (size > 0)
+		in_block = 1;
+
 	} else { /* write or continue writing file contents */
 	    nbytes = size > TAR_BLOCKSIZE ? TAR_BLOCKSIZE : size;
 	    nwritten = fwrite(block, 1, nbytes, outfile);
@@ -210,7 +218,7 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 	unsigned int type, size, pad, compr = 1;
 	off_t off;
 	struct cli_dbinfo *db;
-	unsigned char hash[32];
+	char hash[32];
 
     cli_dbgmsg("in cli_tgzload()\n");
 
@@ -571,7 +579,7 @@ int cl_cvdverify(const char *file)
 {
 	struct cl_engine *engine;
 	FILE *fs;
-	int ret;
+	int ret, dbtype = 0;
 
 
     if((fs = fopen(file, "rb")) == NULL) {
@@ -586,7 +594,12 @@ int cl_cvdverify(const char *file)
     }
     engine->cb_stats_submit = NULL; /* Don't submit stats if we're just verifying a CVD */
 
-    ret = cli_cvdload(fs, engine, NULL, CL_DB_STDOPT | CL_DB_PUA, !!cli_strbcasestr(file, ".cld"), file, 1);
+    if (!!cli_strbcasestr(file, ".cld"))
+	dbtype = 1;
+    else if (!!cli_strbcasestr(file, ".cud"))
+	dbtype = 2;
+
+    ret = cli_cvdload(fs, engine, NULL, CL_DB_STDOPT | CL_DB_PUA, dbtype, file, 1);
 
     cl_engine_free(engine);
     fclose(fs);
@@ -658,10 +671,10 @@ int cli_cvdload(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigne
     }
 
     if(cvd.fl > cl_retflevel()) {
-	cli_warnmsg("***********************************************************\n");
-	cli_warnmsg("***  This version of the ClamAV engine is outdated.     ***\n");
-	cli_warnmsg("***   Read http://www.clamav.net/documentation.html     ***\n");
-	cli_warnmsg("***********************************************************\n");
+	cli_warnmsg("*******************************************************************\n");
+	cli_warnmsg("***  This version of the ClamAV engine is outdated.             ***\n");
+	cli_warnmsg("***   Read https://www.clamav.net/documents/installing-clamav   ***\n");
+	cli_warnmsg("*******************************************************************\n");
     }
 
     cfd = fileno(fs);
