@@ -718,6 +718,8 @@ static void handle_df(const yystype *tokens, size_t start, struct decode_result
 	if(!str)
 		return;
 	len = strlen(str);
+	if(!len)
+		return;
 	clast = str[len-1] - '0';
 
 	str[len-1] = '\0';
@@ -869,7 +871,7 @@ void cli_js_parse_done(struct parser_state* state)
 	}
 	if (end != '\0')
 		cli_js_process_buffer(state, &end, 1);
-	/* close remaining paranthesis */
+	/* close remaining parenthesis */
 	for (i=0;i<tokens->cnt;i++) {
 		if (tokens->data[i].type == TOK_PAR_OPEN)
 			par_balance++;
@@ -885,7 +887,7 @@ void cli_js_parse_done(struct parser_state* state)
 		}
 	}
 
-	/* we had to close unfinished strings, paranthesis,
+	/* we had to close unfinished strings, parenthesis,
 	 * so that the folders/decoders can run properly */
 	run_folders(&state->tokens);
 	run_decoders(state);
