@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015-2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -40,10 +41,7 @@
 #endif
 #include <stddef.h>
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
+#include "clamav.h"
 #include "others.h"
 #include "str.h"
 #include "readdb.h"
@@ -61,7 +59,7 @@
 #ifdef DEBUGMPOOL
 #define spam(...) cli_warnmsg( __VA_ARGS__)
 #else
-static inline void spam(const char *fmt, ...) { fmt = fmt; } /* gcc STFU */
+static inline void spam(const char *fmt, ...) { UNUSEDPARAM(fmt); }
 #endif
 
 #include "mpool.h"
@@ -366,6 +364,7 @@ static const unsigned int fragsz[] = {
   134217728,
 };
 #endif
+
 #define FRAGSBITS (sizeof(fragsz)/sizeof(fragsz[0]))
 
 struct MPMAP {
@@ -617,7 +616,7 @@ void *mpool_malloc(struct MP *mp, size_t size) {
 
   /*  check_all(mp); */
   if (!size || sbits == FRAGSBITS) {
-    cli_errmsg("mpool_malloc(): Attempt to allocate %lu bytes. Please report to http://bugs.clamav.net\n", (unsigned long) size);
+    cli_errmsg("mpool_malloc(): Attempt to allocate %lu bytes. Please report to https://bugzilla.clamav.net\n", (unsigned long) size);
     return NULL;
   }
 
@@ -626,7 +625,11 @@ void *mpool_malloc(struct MP *mp, size_t size) {
     struct FRAG *fold = f;
     mp->avail[sbits] = f->u.next.ptr;
     /* we always have enough space for this, align_increase ensured that */
+#ifdef _WIN64
+    f = (struct FRAG*)(alignto((unsigned long long)f + FRAG_OVERHEAD, align)-FRAG_OVERHEAD);
+#else
     f = (struct FRAG*)(alignto((unsigned long)f + FRAG_OVERHEAD, align)-FRAG_OVERHEAD);
+#endif
     f->u.a.sbits = sbits;
     f->u.a.padding = (char*)f - (char*)fold;
 #ifdef CL_DEBUG
@@ -638,7 +641,7 @@ void *mpool_malloc(struct MP *mp, size_t size) {
   }
 
   if (!(needed = from_bits(sbits))) {
-    cli_errmsg("mpool_malloc(): Attempt to allocate %lu bytes. Please report to http://bugs.clamav.net\n", (unsigned long) size);
+    cli_errmsg("mpool_malloc(): Attempt to allocate %lu bytes. Please report to https://bugzilla.clamav.net\n", (unsigned long) size);
     return NULL;
   }
 
@@ -719,7 +722,7 @@ void *mpool_realloc(struct MP *mp, void *ptr, size_t size) {
   if (!ptr) return mpool_malloc(mp, size);
 
   if(!size || !(csize = from_bits(f->u.a.sbits))) {
-    cli_errmsg("mpool_realloc(): Attempt to allocate %lu bytes. Please report to http://bugs.clamav.net\n", (unsigned long) size);
+    cli_errmsg("mpool_realloc(): Attempt to allocate %lu bytes. Please report to https://bugzilla.clamav.net\n", (unsigned long) size);
     return NULL;
   }
   csize -= FRAG_OVERHEAD + f->u.a.padding;
@@ -743,8 +746,8 @@ void *mpool_realloc2(struct MP *mp, void *ptr, size_t size) {
     return NULL;
 }
 
-unsigned char *cli_mpool_hex2str(mpool_t *mp, const char *hex) {
-    unsigned char *str;
+char *cli_mpool_hex2str(mpool_t *mp, const char *hex) {
+    char *str;
     size_t len = strlen((const char*)hex);
 
     if (len&1) {
@@ -757,7 +760,7 @@ unsigned char *cli_mpool_hex2str(mpool_t *mp, const char *hex) {
 	cli_errmsg("cli_mpool_hex2str(): Can't allocate memory (%lu bytes).\n", (unsigned long)(len/2 + 1));
 	return NULL;
     }
-    if (cli_hex2str_to(hex, (char*)str, len) == -1) {
+    if (cli_hex2str_to(hex, str, len) == -1) {
 	mpool_free(mp, str);
 	return NULL;
     }
@@ -770,7 +773,7 @@ char *cli_mpool_strdup(mpool_t *mp, const char *s) {
   size_t strsz;
 
   if(s == NULL) {
-    cli_errmsg("cli_mpool_strdup(): s == NULL. Please report to http://bugs.clamav.net\n");
+    cli_errmsg("cli_mpool_strdup(): s == NULL. Please report to https://bugzilla.clamav.net\n");
     return NULL;
   }
 
@@ -783,6 +786,25 @@ char *cli_mpool_strdup(mpool_t *mp, const char *s) {
   return alloc;
 }
 
+char *cli_mpool_strndup(mpool_t *mp, const char *s, size_t n) {
+  char *alloc;
+  size_t strsz;
+
+  if(s == NULL) {
+    cli_errmsg("cli_mpool_strndup(): s == NULL. Please report to https://bugzilla.clamav.net\n");
+    return NULL;
+  }
+
+  strsz = cli_strnlen(s, n) + 1;
+  alloc = mpool_malloc(mp, strsz);
+  if(!alloc)
+    cli_errmsg("cli_mpool_strndup(): Can't allocate memory (%lu bytes).\n", (unsigned long) strsz);
+  else
+    memcpy(alloc, s, strsz-1);
+  alloc[strsz-1] = '\0';
+  return alloc;
+}
+
 /* #define EXPAND_PUA */
 char *cli_mpool_virname(mpool_t *mp, const char *virname, unsigned int official) {
   char *newname, *pt;
