@@ -79,24 +79,6 @@ static const struct key_entry msxml_keys[] = {
 };
 static size_t num_msxml_keys = sizeof(msxml_keys) / sizeof(struct key_entry);
 
-enum msxml_state {
-    MSXML_STATE_NORMAL = 0,
-    MSXML_STATE_ENTITY_START_1,
-    MSXML_STATE_ENTITY_START_2,
-    MSXML_STATE_ENTITY_HEX,
-    MSXML_STATE_ENTITY_DEC,
-    MSXML_STATE_ENTITY_CLOSE,
-    MSXML_STATE_ENTITY_NONE
-};
-
-struct msxml_cbdata {
-    enum msxml_state state;
-    fmap_t *map;
-    const unsigned char *window;
-    off_t winpos, mappos;
-    size_t winsize;
-};
-
 static inline size_t msxml_read_cb_new_window(struct msxml_cbdata *cbdata)
 {
     const unsigned char *new_window = NULL;
@@ -263,7 +245,7 @@ int cli_scanmsxml(cli_ctx *ctx)
 
     reader = xmlReaderForIO(msxml_read_cb, NULL, &cbdata, "msxml.xml", NULL, CLAMAV_MIN_XMLREADER_FLAGS);
     if (!reader) {
-        cli_dbgmsg("cli_scanmsxml: cannot intialize xmlReader\n");
+        cli_dbgmsg("cli_scanmsxml: cannot initialize xmlReader\n");
 
 #if HAVE_JSON
         ret = cli_json_parse_error(ctx->wrkproperty, "OOXML_ERROR_XML_READER_IO");
@@ -271,7 +253,7 @@ int cli_scanmsxml(cli_ctx *ctx)
         return ret; // libxml2 failed!
     }
 
-    ret = cli_msxml_parse_document(ctx, reader, msxml_keys, num_msxml_keys, 1);
+    ret = cli_msxml_parse_document(ctx, reader, msxml_keys, num_msxml_keys, 1, NULL);
 
     xmlTextReaderClose(reader);
     xmlFreeTextReader(reader);
