@@ -1,5 +1,5 @@
 /*
- * Extract component parts of MS XML files (e.g. MS Office 2003 XML Documents)
+ * Extract component parts of various MS XML files (e.g. MS Office 2003 XML Documents)
  * 
  * Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  * Copyright (C) 2007-2013 Sourcefire, Inc.
@@ -30,7 +30,6 @@
 #endif
 
 #include "others.h"
-#include "json_api.h"
 
 #ifdef _WIN32
 #ifndef LIBXML_WRITER_ENABLED
@@ -43,6 +42,12 @@
 #define MSXML_RECLEVEL_MAX 20
 #define MSXML_JSON_STRLEN_MAX 128
 
+/* reader usage flags */
+#define MSXML_FLAG_JSON  0x1
+#define MSXML_FLAG_WALK  0x2
+
+struct msxml_ictx;
+
 struct attrib_entry {
     const char *key;
     const char *value;
@@ -54,37 +59,35 @@ struct key_entry {
 #define MSXML_IGNORE_ELEM     0x1
 #define MSXML_SCAN_CB         0x2
 #define MSXML_SCAN_B64        0x4
+#define MSXML_COMMENT_CB      0x8
 /* where */
-#define MSXML_JSON_ROOT       0x8
-#define MSXML_JSON_WRKPTR     0x10
-#define MSXML_JSON_MULTI      0x20
+#define MSXML_JSON_ROOT       0x10
+#define MSXML_JSON_WRKPTR     0x20
+#define MSXML_JSON_MULTI      0x40
 
 #define MSXML_JSON_TRACK (MSXML_JSON_ROOT | MSXML_JSON_WRKPTR)
 /* what */
-#define MSXML_JSON_COUNT      0x40
-#define MSXML_JSON_VALUE      0x80
-#define MSXML_JSON_ATTRIB     0x100
+#define MSXML_JSON_COUNT      0x100
+#define MSXML_JSON_VALUE      0x200
+#define MSXML_JSON_ATTRIB     0x400
 
     const char *key;
     const char *name;
     uint32_t type;
 };
 
-typedef int (*msxml_scan_cb)(int fd, cli_ctx *ctx, int num_attribs, struct attrib_entry *attribs);
+typedef int (*msxml_scan_cb)(int fd, cli_ctx *ctx, int num_attribs, struct attrib_entry *attribs, void *cbdata);
+typedef int (*msxml_comment_cb)(const char *comment, cli_ctx *ctx, void *wrkjobj, void *cbdata);
 
 struct msxml_ctx {
-    cli_ctx *ctx;
     msxml_scan_cb scan_cb;
-    const struct key_entry *keys;
-    size_t num_keys;
-
-#if HAVE_JSON
-    json_object *root;
-    int mode, toval;
-#endif
+    void *scan_data;
+    msxml_comment_cb comment_cb;
+    void *comment_data;
+    struct msxml_ictx *ictx;
 };
 
-int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct key_entry *keys, const size_t num_keys, int mode, msxml_scan_cb scan_cb);
+int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct key_entry *keys, const size_t num_keys, uint32_t flags, struct msxml_ctx *mxctx);
 
 #endif /* HAVE_LIBXML2 */
 
