@@ -80,7 +80,7 @@ static int unz(const uint8_t *src, uint32_t csize, uint32_t usize, uint16_t meth
   if((of = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRUSR|S_IWUSR))==-1) {
     cli_warnmsg("cli_unzip: failed to create temporary file %s\n", tempfile);
     if(!tmpd) free(tempfile);
-    return CL_ECREAT;
+    return CL_ETMPFILE;
   }
   switch (method) {
   case ALG_STORED:
@@ -333,7 +333,7 @@ static inline void zinitkey(uint32_t key[3], struct cli_pwdb *password)
 {
     int i;
 
-    /* intialize keys, these are specified but the zip specification */
+    /* initialize keys, these are specified but the zip specification */
     key[0] = 305419896L;
     key[1] = 591751049L;
     key[2] = 878082192L;
@@ -432,7 +432,7 @@ static inline int zdecrypt(const uint8_t *src, uint32_t csize, uint32_t usize, c
 	    if((of = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRUSR|S_IWUSR))==-1) {
 		cli_warnmsg("cli_unzip: decrypt - failed to create temporary file %s\n", tempfile);
 		if(!tmpd) free(tempfile);
-		return CL_ECREAT;
+		return CL_ETMPFILE;
 	    }
 
 	    for (i = 12; i < csize; i++) {
@@ -504,6 +504,7 @@ static unsigned int lhdr(fmap_t *map, uint32_t loff,uint32_t zsize, unsigned int
   const uint8_t *lh, *zip;
   char name[256];
   uint32_t csize, usize;
+  int virus_found = 0;
 
   if(!(lh = fmap_need_off(map, loff, SIZEOF_LH))) {
       cli_dbgmsg("cli_unzip: lh - out of file\n");
@@ -540,8 +541,10 @@ static unsigned int lhdr(fmap_t *map, uint32_t loff,uint32_t zsize, unsigned int
   /* ZMDfmt virname:encrypted(0-1):filename(exact|*):usize(exact|*):csize(exact|*):crc32(exact|*):method(exact|*):fileno(exact|*):maxdepth(exact|*) */
 
   if(cli_matchmeta(ctx, name, LH_csize, LH_usize, (LH_flags & F_ENCR)!=0, fc, LH_crc32, NULL) == CL_VIRUS) {
-    *ret = CL_VIRUS;
-    return 0;
+      *ret = CL_VIRUS;
+      if (!SCAN_ALL)
+          return 0;
+      virus_found = 1;
   }
 
   if(LH_flags & F_MSKED) {
@@ -553,10 +556,12 @@ static unsigned int lhdr(fmap_t *map, uint32_t loff,uint32_t zsize, unsigned int
 
   if(detect_encrypted && (LH_flags & F_ENCR) && DETECT_ENCRYPTED) {
     cli_dbgmsg("cli_unzip: Encrypted files found in archive.\n");
-    cli_append_virus(ctx, "Heuristics.Encrypted.Zip");
-    *ret = CL_VIRUS;
-    fmap_unneed_off(map, loff, SIZEOF_LH);
-    return 0;
+    *ret = cli_append_virus(ctx, "Heuristics.Encrypted.Zip");
+    if ((*ret == CL_VIRUS && !SCAN_ALL) || *ret != CL_CLEAN) {
+        fmap_unneed_off(map, loff, SIZEOF_LH);
+        return 0;
+    }
+    virus_found = 1;
   }
  
   if(LH_flags & F_USEDD) {
@@ -595,6 +600,9 @@ static unsigned int lhdr(fmap_t *map, uint32_t loff,uint32_t zsize, unsigned int
       zsize-=csize;
   }
 
+  if (virus_found != 0)
+      *ret = CL_VIRUS;
+
   fmap_unneed_off(map, loff, SIZEOF_LH); /* unneed now. block is guaranteed to exists till the next need */
   if(LH_flags & F_USEDD) {
       if(zsize<12) {
@@ -620,6 +628,7 @@ static unsigned int chdr(fmap_t *map, uint32_t coff, uint32_t zsize, unsigned in
   char name[256];
   int last = 0;
   const uint8_t *ch;
+  int virus_found = 0;
 
   if(!(ch = fmap_need_off(map, coff, SIZEOF_CH)) || CH_magic != 0x02014b50) {
       if(ch) fmap_unneed_ptr(map, ch, SIZEOF_CH);
@@ -636,7 +645,7 @@ static unsigned int chdr(fmap_t *map, uint32_t coff, uint32_t zsize, unsigned in
   }
 
   name[0]='\0';
-  if((cli_debug_flag && !last) || requests) {
+  if(!last) {
       unsigned int size = (CH_flen>=sizeof(name))?sizeof(name)-1:CH_flen;
       const char *src = fmap_need_off_once(map, coff, size);
       if(src) {
@@ -647,6 +656,10 @@ static unsigned int chdr(fmap_t *map, uint32_t coff, uint32_t zsize, unsigned in
   }
   coff+=CH_flen;
 
+  /* requests do not supply a ctx; also prevent multiple scans */
+  if(ctx && cli_matchmeta(ctx, name, CH_csize, CH_usize, (CH_flags & F_ENCR)!=0, fc, CH_crc32, NULL) == CL_VIRUS)
+    virus_found = 1;
+
   if(zsize-coff<=CH_elen && !last) {
     cli_dbgmsg("cli_unzip: ch - extra out of file\n");
     last=1;
@@ -682,6 +695,8 @@ static unsigned int chdr(fmap_t *map, uint32_t coff, uint32_t zsize, unsigned in
       }
   }
 
+  if (virus_found == 1)
+      *ret = CL_VIRUS;
   fmap_unneed_ptr(map, ch, SIZEOF_CH);
   return (last?0:coff);
 }
@@ -730,7 +745,7 @@ int cli_unzip(cli_ctx *ctx) {
 
   if(coff) {
       cli_dbgmsg("cli_unzip: central @%x\n", coff);
-      while(ret==CL_CLEAN && (coff=chdr(map, coff, fsize, &fu, fc+1, &ret, ctx, tmpd, NULL))) {
+      while((coff=chdr(map, coff, fsize, &fu, fc+1, &ret, ctx, tmpd, NULL))) {
 	  fc++;
 	  if (ctx->engine->maxfiles && fu>=ctx->engine->maxfiles) {
 	      cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->engine->maxfiles);
@@ -741,9 +756,17 @@ int cli_unzip(cli_ctx *ctx) {
               ret=CL_ETIMEOUT;
           }
 #endif
-
+          if (ret != CL_CLEAN) {
+              if (ret == CL_VIRUS && SCAN_ALL) {
+                  ret = CL_CLEAN;
+                  virus_found = 1;
+              } else
+                  break;
+          }
       }
   } else cli_dbgmsg("cli_unzip: central not found, using localhdrs\n");
+  if (virus_found == 1)
+      ret = CL_VIRUS;
   if(fu<=(fc/4)) { /* FIXME: make up a sane ratio or remove the whole logic */
     fc = 0;
     while (ret==CL_CLEAN && lhoff<fsize && (coff=lhdr(map, lhoff, fsize-lhoff, &fu, fc+1, NULL, &ret, ctx, tmpd, 1, zip_scan_cb))) {
